from cement import Controller, ex
from cement.utils import fs
from benchgrape.core.db import TestDataMapper
import csv


class TestData(Controller):
    class Meta:
        label = 'test-data'
        stacked_type = 'nested'
        stacked_on = 'base'

    @ex(help='Load Test Data from JSON-Config File generated by '
             'chatgrape\'s benchmark_setup command.',
        arguments=[
            (['token_file'], {
                'help': 'Token File to use (csv)',
                'action': 'store'
            })
        ],
    )
    def load(self):
        fname = fs.abspath(self.app.pargs.token_file)
        self.app.log.info('loading tokens file from: %s' % fname)
        tokens = []

        with open(fname, 'r') as fhandle:
            readCSV = csv.reader(fhandle, delimiter=',')
            for row in readCSV:
                tokens.append(row[0])

        TestDataMapper(self.app.db, self.app.log).drop_db()
        TestDataMapper(self.app.db, self.app.log).init_db()
        TestDataMapper(self.app.db, self.app.log).sync_db(tokens)

    @ex(help='Purge test data in the BenchGrape DB. '
             'This also reset the internal data structure.')
    def purge(self):
        TestDataMapper(self.app.db, self.app.log).drop_db()
